@@ -80,16 +80,32 @@ module Lint
       target_version "2.3.9" do # 2.4-rc6
         # Non-nested array with pairs
         assert_equal 0, r.zcard("foo")
+
         assert_equal 2, r.zadd("foo", [1, "s1", 2, "s2"])
+        assert_equal 2, r.zcard("foo")
+
         assert_equal 1, r.zadd("foo", [4, "s1", 5, "s2", 6, "s3"])
         assert_equal 3, r.zcard("foo")
+
         r.del "foo"
 
         # Nested array with pairs
         assert_equal 0, r.zcard("foo")
+
         assert_equal 2, r.zadd("foo", [[1, "s1"], [2, "s2"]])
+        assert_equal 2, r.zcard("foo")
+
         assert_equal 1, r.zadd("foo", [[4, "s1"], [5, "s2"], [6, "s3"]])
         assert_equal 3, r.zcard("foo")
+
+        r.del "foo"
+
+        # Empty array
+        assert_equal 0, r.zcard("foo")
+
+        assert_equal 0, r.zadd("foo", [])
+        assert_equal 0, r.zcard("foo")
+
         r.del "foo"
 
         # Wrong number of arguments
@@ -179,8 +195,16 @@ module Lint
         r.zadd("foo", 3, "s3")
 
         assert_equal 3, r.zcard("foo")
+
+        assert_equal 0, r.zrem("foo", [])
+        assert_equal 3, r.zcard("foo")
+
         assert_equal 1, r.zrem("foo", ["s1", "aaa"])
+        assert_equal 2, r.zcard("foo")
+
         assert_equal 0, r.zrem("foo", ["bbb", "ccc", "ddd"])
+        assert_equal 2, r.zcard("foo")
+
         assert_equal 1, r.zrem("foo", ["eee", "s3"])
         assert_equal 1, r.zcard("foo")
       end
@@ -231,6 +255,45 @@ module Lint
       assert_equal [["s1", -Float::INFINITY], ["s2", +Float::INFINITY]], r.zrange("bar", 0, 1, withscores: true)
     end
 
+    def test_zrange_with_byscore
+      target_version("6.2") do
+        r.zadd "foo", 1, "s1"
+        r.zadd "foo", 2, "s2"
+        r.zadd "foo", 3, "s3"
+
+        assert_equal ["s2", "s3"], r.zrange("foo", 2, 3, byscore: true)
+        assert_equal ["s2", "s1"], r.zrange("foo", 2, 1, byscore: true, rev: true)
+      end
+    end
+
+    def test_zrange_with_bylex
+      target_version("6.2") do
+        r.zadd "foo", 0, "aaren"
+        r.zadd "foo", 0, "abagael"
+        r.zadd "foo", 0, "abby"
+        r.zadd "foo", 0, "abbygail"
+
+        assert_equal %w[aaren abagael abby abbygail], r.zrange("foo", "[a", "[a\xff", bylex: true)
+        assert_equal %w[aaren abagael], r.zrange("foo", "[a", "[a\xff", bylex: true, limit: [0, 2])
+        assert_equal %w[abby abbygail], r.zrange("foo", "(abb", "(abb\xff", bylex: true)
+        assert_equal %w[abbygail], r.zrange("foo", "(abby", "(abby\xff", bylex: true)
+      end
+    end
+
+    def test_zrangestore
+      target_version("6.2") do
+        r.zadd "foo", 1, "s1"
+        r.zadd "foo", 2, "s2"
+        r.zadd "foo", 3, "s3"
+
+        assert_equal 2, r.zrangestore("bar", "foo", 0, 1)
+        assert_equal ["s1", "s2"], r.zrange("bar", 0, -1)
+
+        assert_equal 2, r.zrangestore("baz", "foo", 2, 3, by_score: true)
+        assert_equal ["s2", "s3"], r.zrange("baz", 0, -1)
+      end
+    end
+
     def test_zrevrange
       r.zadd "foo", 1, "s1"
       r.zadd "foo", 2, "s2"
@@ -480,6 +543,52 @@ module Lint
       assert_equal 2, r.zcount('foo', 2, 3)
     end
 
+    def test_zunion
+      target_version("6.2") do
+        r.zadd 'foo', 1, 's1'
+        r.zadd 'foo', 2, 's2'
+        r.zadd 'bar', 3, 's1'
+        r.zadd 'bar', 5, 's3'
+
+        assert_equal %w[s2 s1 s3], r.zunion('foo', 'bar')
+        assert_equal [['s2', 2.0], ['s1', 4.0], ['s3', 5.0]], r.zunion('foo', 'bar', with_scores: true)
+      end
+    end
+
+    def test_zunion_with_weights
+      target_version("6.2") do
+        r.zadd 'foo', 1, 's1'
+        r.zadd 'foo', 2, 's2'
+        r.zadd 'foo', 3, 's3'
+        r.zadd 'bar', 20, 's2'
+        r.zadd 'bar', 30, 's3'
+        r.zadd 'bar', 100, 's4'
+
+        assert_equal %w[s1 s2 s3 s4], r.zunion('foo', 'bar')
+        assert_equal [['s1', 1.0], ['s2', 22.0], ['s3', 33.0], ['s4', 100.0]], r.zunion('foo', 'bar', with_scores: true)
+
+        assert_equal %w[s1 s2 s3 s4], r.zunion('foo', 'bar', weights: [10, 1])
+        assert_equal [['s1', 10.0], ['s2', 40.0], ['s3', 60.0], ['s4', 100.0]], r.zunion('foo', 'bar', weights: [10, 1], with_scores: true)
+      end
+    end
+
+    def test_zunion_with_aggregate
+      target_version("6.2") do
+        r.zadd 'foo', 1, 's1'
+        r.zadd 'foo', 20, 's2'
+        r.zadd 'foo', 3, 's3'
+        r.zadd 'bar', 2, 's2'
+        r.zadd 'bar', 30, 's3'
+        r.zadd 'bar', 100, 's4'
+
+        assert_equal %w[s1 s2 s3 s4], r.zunion('foo', 'bar', aggregate: :min)
+        assert_equal [['s1', 1.0], ['s2', 2.0], ['s3', 3.0], ['s4', 100.0]], r.zunion('foo', 'bar', aggregate: :min, with_scores: true)
+
+        assert_equal %w[s1 s2 s3 s4], r.zunion('foo', 'bar', aggregate: :max)
+        assert_equal [['s1', 1.0], ['s2', 20.0], ['s3', 30.0], ['s4', 100.0]], r.zunion('foo', 'bar', aggregate: :max, with_scores: true)
+      end
+    end
+
     def test_zunionstore
       r.zadd 'foo', 1, 's1'
       r.zadd 'bar', 2, 's2'
@@ -525,6 +634,37 @@ module Lint
       assert_equal 5, r.zunionstore('{1}baz', %w[{1}foo {1}bar])
     end
 
+    def test_zdiff
+      target_version("6.2") do
+        r.zadd 'foo', 1, 's1'
+        r.zadd 'foo', 2, 's2'
+        r.zadd 'bar', 3, 's1'
+        r.zadd 'bar', 5, 's3'
+
+        assert_equal [], r.zdiff('foo', 'foo')
+        assert_equal ['s1', 's2'], r.zdiff('foo')
+
+        assert_equal ['s2'], r.zdiff('foo', 'bar')
+        assert_equal [['s2', 2.0]], r.zdiff('foo', 'bar', with_scores: true)
+      end
+    end
+
+    def test_zdiffstore
+      target_version("6.2") do
+        r.zadd 'foo', 1, 's1'
+        r.zadd 'foo', 2, 's2'
+        r.zadd 'bar', 3, 's1'
+        r.zadd 'bar', 5, 's3'
+
+        assert_equal 0, r.zdiffstore('baz', ['foo', 'foo'])
+        assert_equal 2, r.zdiffstore('baz', ['foo'])
+        assert_equal ['s1', 's2'], r.zrange('baz', 0, -1)
+
+        assert_equal 1, r.zdiffstore('baz', ['foo', 'bar'])
+        assert_equal ['s2'], r.zrange('baz', 0, -1)
+      end
+    end
+
     def test_zinter
       target_version("6.2") do
         r.zadd 'foo', 1, 's1'
